@@ -1,3 +1,10 @@
+use crate::{
+    melpow,
+    melvm::{Address, CovenantEnv},
+    BlockHeight, CoinData, CoinDataHeight, CoinID, CoinValue, Denom, NetID, stake::StakeDoc, state::melmint, State,
+    StateError, Transaction, TxHash, TxKind,
+};
+
 use std::convert::TryInto;
 
 use dashmap::DashMap;
@@ -5,15 +12,6 @@ use parking_lot::Mutex;
 use rayon::iter::{IntoParallelRefIterator, ParallelIterator};
 use tmelcrypt::HashVal;
 
-use crate::{
-    melpow,
-    melvm::{Address, CovenantEnv},
-    BlockHeight, CoinData, CoinDataHeight, CoinID, CoinValue, Denom, NetID, StakeDoc, State,
-    StateError, Transaction, TxHash, TxKind,
-};
-
-use super::melmint;
-
 /// A mutable "handle" to a particular State. Can be "committed" like a database transaction.
 pub(crate) struct StateHandle<'a> {
     state: &'a mut State,
@@ -59,11 +57,14 @@ impl<'a> StateHandle<'a> {
 
     /// Applies a batch of transactions, returning an error if any of them fail. Consumes and re-returns the handle; if any fail the handle is gone.
     pub fn apply_tx_batch(mut self, txx: &[Transaction]) -> Result<Self, StateError> {
-        for tx in txx.iter() {
-            if tx.kind == TxKind::Faucet {
+        let was_batch_successful: Result<(), crate::state::StateError> = txx.iter().try_for_each(|tx| {
+            let is_transaction_a_faucet: bool = tx.kind == TxKind::Faucet;
+
+            if is_transaction_a_faucet {
                 let pseudocoin = faucet_dedup_pseudocoin(tx.hash_nosigs());
+
                 if self.state.coins.get(&pseudocoin).0.is_some() {
-                    return Err(StateError::DuplicateTx);
+                    Err(StateError::DuplicateTx)
                 } else {
                     self.state.coins.insert(
                         pseudocoin,
@@ -77,56 +78,88 @@ impl<'a> StateHandle<'a> {
                             height: 0.into(),
                         },
                     );
+
+                    Ok(())
                 }
+            } else if !tx.is_well_formed() {
+                Err(StateError::MalformedTx)
+            } else if is_transaction_a_faucet && self.state.network == NetID::Mainnet {
+                Err(StateError::UnbalancedInOut)
+            } else {
+                self.transactions_cache.insert(tx.hash_nosigs(), tx.clone());
+
+                self.apply_tx_fees(tx)
             }
-            if !tx.is_well_formed() {
-                return Err(StateError::MalformedTx);
-            }
-            if tx.kind == TxKind::Faucet && self.state.network == NetID::Mainnet {
-                return Err(StateError::UnbalancedInOut);
-            }
-            self.transactions_cache.insert(tx.hash_nosigs(), tx.clone());
-            self.apply_tx_fees(tx)?;
+        });
+
+        match was_batch_successful {
+            Ok(()) => {
+                // apply specials in parallel
+                txx.par_iter()
+                    .filter(|tx| tx.kind != TxKind::Normal && tx.kind != TxKind::Faucet)
+                    .map(|tx| self.apply_tx_special(tx))
+                    .collect::<Result<_, _>>()?;
+                // apply outputs in parallel
+                txx.par_iter().for_each(|tx| self.apply_tx_outputs(tx));
+                // apply inputs in parallel
+                txx.par_iter()
+                    .map(|tx| self.apply_tx_inputs(tx))
+                    .collect::<Result<_, _>>()?;
+
+                Ok(self)
+            },
+            Err(error) => Err(error)
         }
-        // apply specials in parallel
-        txx.par_iter()
-            .filter(|tx| tx.kind != TxKind::Normal && tx.kind != TxKind::Faucet)
-            .map(|tx| self.apply_tx_special(tx))
-            .collect::<Result<_, _>>()?;
-        // apply outputs in parallel
-        txx.par_iter().for_each(|tx| self.apply_tx_outputs(tx));
-        // apply inputs in parallel
-        txx.par_iter()
-            .map(|tx| self.apply_tx_inputs(tx))
-            .collect::<Result<_, _>>()?;
-        Ok(self)
     }
 
     /// Commits all the changes in this handle, at once.
     pub fn commit(self) {
         // commit coins
-        for (k, v) in self.coin_cache {
-            if let Some(v) = v {
-                self.state.coins.insert(k, v);
+        self.coin_cache.into_iter().for_each(|(key, value)| {
+            if let Some(value) = value {
+                self.state.coins.insert(key, value);
             } else {
-                self.state.coins.delete(&k);
+                self.state.coins.delete(&key);
             }
-        }
+        });
+
         // commit txx
-        for (k, v) in self.transactions_cache {
-            self.state.transactions.insert(k, v);
-        }
+        self.transactions_cache.into_iter().for_each(|(key, value)| {
+            self.state.transactions.insert(key, value);
+        });
+
         // commit fees
         self.state.fee_pool = self.fee_pool_cache;
         self.state.tips = self.tips_cache;
+
         // commit stakes
-        for (k, v) in self.stakes_cache {
-            self.state.stakes.insert(k, v);
-        }
+        self.stakes_cache.into_iter().for_each(|(key, value)| {
+            self.state.stakes.insert(key, value);
+        });
+
         self.state.dosc_speed = *self.dosc_speed_cache.lock()
     }
 
     fn apply_tx_inputs(&self, tx: &Transaction) -> Result<(), StateError> {
+        // let mut output: Vec<u8> = Vec::new();
+        // // go through output
+        // let was_encoding_successful: Result<(), EncodeError> = ops.iter().try_for_each(|op| {
+        //     match op.encode() {
+        //         Ok(data) => {
+        //             Ok(output.extend_from_slice(&data))
+        //         },
+        //         Err(error) => Err(error),
+        //     }
+        // });
+        //
+        // match was_encoding_successful {
+        //     Ok(()) => Ok(Covenant(output)),
+        //     Err(error) => Err(error),
+        // }
+
+
+
+
         let scripts = tx.script_as_map();
         // build a map of input coins
         let mut in_coins: imbl::HashMap<Denom, CoinValue> = imbl::HashMap::new();
@@ -137,90 +170,122 @@ impl<'a> StateHandle<'a> {
             .get(&BlockHeight(self.state.height.0.saturating_sub(1)))
             .0
             .unwrap_or_else(|| self.state.clone().seal(None).header());
+
+
         // iterate through the inputs
-        for (spend_idx, coin_id) in tx.inputs.iter().enumerate() {
+        let were_inputs_successful: Result<(), StateError> = tx.inputs.iter().enumerate().try_for_each(|(spend_idx, coin_id)| {
             if self.get_stake(coin_id.txhash).is_some() && coin_id.index == 0 {
                 // only the first output is locked
-                return Err(StateError::CoinLocked);
-            }
-            let coin_data = self.get_coin(*coin_id);
-            match coin_data {
-                None => return Err(StateError::NonexistentCoin(*coin_id)),
-                Some(coin_data) => {
-                    log::trace!(
+                Err(StateError::CoinLocked)
+            } else {
+                let coin_data = self.get_coin(*coin_id);
+
+                match coin_data {
+                    None => Err(StateError::NonexistentCoin(*coin_id)),
+                    Some(coin_data) => {
+                        log::trace!(
                         "coin_data {:?} => {:?} for txid {:?}",
                         coin_id,
                         coin_data,
                         tx.hash_nosigs()
                     );
-                    let script = scripts
-                        .get(&coin_data.coin_data.covhash)
-                        .ok_or(StateError::NonexistentScript(coin_data.coin_data.covhash))?;
-                    if !script.check(
-                        tx,
-                        CovenantEnv {
-                            parent_coinid: coin_id,
-                            parent_cdh: &coin_data,
-                            spender_index: spend_idx as u8,
-                            last_header: &last_header,
-                        },
-                    ) {
-                        return Err(StateError::ViolatesScript(coin_data.coin_data.covhash));
+                        let script = scripts
+                            .get(&coin_data.coin_data.covhash)
+                            .ok_or(StateError::NonexistentScript(coin_data.coin_data.covhash))?;
+
+                        let was_check_an_error: bool = !script.check(
+                            tx,
+                            CovenantEnv {
+                                parent_coinid: coin_id,
+                                parent_cdh: &coin_data,
+                                spender_index: spend_idx as u8,
+                                last_header: &last_header,
+                            },
+                        );
+
+                        if was_check_an_error {
+                            Err(StateError::ViolatesScript(coin_data.coin_data.covhash))
+                        } else {
+                            self.del_coin(*coin_id);
+
+                            in_coins.insert(
+                                coin_data.coin_data.denom,
+                                in_coins
+                                    .get(&coin_data.coin_data.denom)
+                                    .cloned()
+                                    .unwrap_or_else(|| 0u128.into())
+                                    + coin_data.coin_data.value,
+                            );
+
+                            Ok(())
+                        }
                     }
-                    self.del_coin(*coin_id);
-                    in_coins.insert(
-                        coin_data.coin_data.denom,
-                        in_coins
-                            .get(&coin_data.coin_data.denom)
-                            .cloned()
-                            .unwrap_or_else(|| 0u128.into())
-                            + coin_data.coin_data.value,
-                    );
                 }
             }
-        }
-        // balance inputs and outputs. ignore outputs with empty cointype (they create a new token kind)
-        let out_coins = tx.total_outputs();
-        if tx.kind != TxKind::Faucet {
-            for (currency, value) in out_coins.iter() {
-                // we skip the created doscs for a DoscMint transaction
-                if tx.kind == TxKind::DoscMint && *currency == Denom::NomDosc {
-                    continue;
+        });
+
+        match were_inputs_successful {
+            Err(error) => Err(error),
+            Ok(()) => {
+                // balance inputs and outputs. ignore outputs with empty cointype (they create a new token kind)
+                let out_coins = tx.total_outputs();
+
+                let is_transaction_not_a_faucet: bool = tx.kind != TxKind::Faucet;
+
+                if is_transaction_not_a_faucet {
+                    out_coins.iter().try_for_each(|(currency, value)| {
+                        // we skip the created doscs for a DoscMint transaction
+                        let is_transaction_a_dosc: bool = tx.kind == TxKind::DoscMint && *currency == Denom::NomDosc;
+
+                        if !is_transaction_a_dosc {
+                            let in_value = in_coins
+                                .get(currency)
+                                .copied()
+                                .unwrap_or_else(|| u128::MAX.into());
+
+                            let unbalanced_check: bool = *currency != Denom::NewCoin && *value != in_value;
+
+                            if unbalanced_check {
+                                log::warn!(
+                                    "unbalanced: {} {:?} in, {} {:?} out",
+                                    in_value,
+                                    currency,
+                                    value,
+                                    currency);
+
+                                Err(StateError::UnbalancedInOut)
+                            } else {
+                                Ok(())
+                            }
+                        } else {
+                            Ok(())
+                        }
+                    })
+                } else {
+                    Ok(())
                 }
-                let in_value = in_coins
-                    .get(currency)
-                    .copied()
-                    .unwrap_or_else(|| u128::MAX.into());
-                if *currency != Denom::NewCoin && *value != in_value {
-                    log::warn!(
-                        "unbalanced: {} {:?} in, {} {:?} out",
-                        in_value,
-                        currency,
-                        value,
-                        currency
-                    );
-                    return Err(StateError::UnbalancedInOut);
-                }
-            }
+            },
         }
-        Ok(())
     }
 
     fn apply_tx_fees(&mut self, tx: &Transaction) -> Result<(), StateError> {
         // fees
         let min_fee = tx.base_fee(self.state.fee_multiplier, 0);
         if tx.fee < min_fee {
-            return Err(StateError::InsufficientFees(min_fee));
+            Err(StateError::InsufficientFees(min_fee))
+        } else {
+            let tips = tx.fee - min_fee;
+            self.tips_cache.0 = self.tips_cache.0.saturating_add(tips.0);
+            self.fee_pool_cache.0 = self.fee_pool_cache.0.saturating_add(min_fee.0);
+
+            Ok(())
         }
-        let tips = tx.fee - min_fee;
-        self.tips_cache.0 = self.tips_cache.0.saturating_add(tips.0);
-        self.fee_pool_cache.0 = self.fee_pool_cache.0.saturating_add(min_fee.0);
-        Ok(())
     }
 
     fn apply_tx_outputs(&self, tx: &Transaction) {
         let height = self.state.height;
-        for (index, coin_data) in tx.outputs.iter().enumerate() {
+
+        tx.outputs.iter().enumerate().for_each(|(index, coin_data)| {
             let mut coin_data = coin_data.clone();
             if coin_data.denom == Denom::NewCoin {
                 coin_data.denom = Denom::Custom(tx.hash_nosigs());
@@ -235,7 +300,7 @@ impl<'a> StateHandle<'a> {
                     CoinDataHeight { coin_data, height },
                 );
             }
-        }
+        });
     }
 
     fn apply_tx_special(&self, tx: &Transaction) -> Result<(), StateError> {
@@ -310,17 +375,21 @@ impl<'a> StateHandle<'a> {
         let curr_epoch = self.state.height.epoch();
         // then we check that the first coin is valid
         let first_coin = tx.outputs.get(0).ok_or(StateError::MalformedTx)?;
-        if first_coin.denom != Denom::Sym {
-            return Err(StateError::MalformedTx);
-        }
+
+        let is_first_coin_not_a_sym: bool = first_coin.denom != Denom::Sym;
+
+        if is_first_coin_not_a_sym {
+            Err(StateError::MalformedTx)
         // then we check consistency
-        if !(stake_doc.e_start > curr_epoch
+        } else if !(stake_doc.e_start > curr_epoch
             && stake_doc.e_post_end > stake_doc.e_start
-            && stake_doc.syms_staked == first_coin.value)
-        {
+            && stake_doc.syms_staked == first_coin.value) {
             self.set_stake(tx.hash_nosigs(), stake_doc);
+
+            Ok(())
+        } else {
+            Ok(())
         }
-        Ok(())
     }
 
     fn get_coin(&self, coin_id: CoinID) -> Option<CoinDataHeight> {
@@ -341,12 +410,12 @@ impl<'a> StateHandle<'a> {
 
     fn get_stake(&self, txhash: TxHash) -> Option<StakeDoc> {
         if let Some(cached_sd) = self.stakes_cache.get(&txhash).as_deref() {
-            return Some(cached_sd).cloned();
+            Some(cached_sd).cloned()
+        } else if let Some(sd) = self.state.stakes.get(&txhash).0 {
+            self.stakes_cache.insert(txhash, sd)
+        } else {
+            None
         }
-        if let Some(sd) = self.state.stakes.get(&txhash).0 {
-            return self.stakes_cache.insert(txhash, sd);
-        }
-        None
     }
 
     fn set_stake(&self, txhash: TxHash, sdoc: StakeDoc) {
@@ -394,4 +463,4 @@ pub(crate) mod tests {
     //     //
     //     // assert!(res.is_ok());
     // }
-}
+}
\ No newline at end of file
