@@ -1,7 +1,10 @@
 use std::fmt::Display;
 
 use arbitrary::Arbitrary;
-use derive_more::{Mul, MulAssign, Add, AddAssign, Display, Div, DivAssign, From, FromStr, Into, Sub, SubAssign};
+use derive_more::{
+    Add, AddAssign, Display, Div, DivAssign, From, FromStr, Into, Mul, MulAssign, Sub, SubAssign,
+    Sum,
+};
 use serde::{Deserialize, Serialize};
 
 use crate::{MICRO_CONVERTER, STAKE_EPOCH};
@@ -30,6 +33,7 @@ use crate::{MICRO_CONVERTER, STAKE_EPOCH};
     DivAssign,
     Mul,
     MulAssign,
+    Sum,
 )]
 #[serde(transparent)]
 pub struct CoinValue(pub u128);
